/*!
  * Generic indexeddb functions for local storage
  * Kitst Config needs a list of TimeZones
  * On first use indexeddb will be set up with UTC, GMT and the configuration strings.
  * An attempt may also be made to load a further list of common TimeZones from kitst.co.uk
  * Hopefully there is a match with the local TimeZone else defaults to GMT.
  * The date structure is a key, value pair. 
  * Key is Time Zone Name and value is a string in the form:
  * Z{DST, 5, 1, 03, 01, 060 }{STD, 5, 1, 10, 02, 000 }
  * == {Which Week? , Day of Week, Month, Hour, Offset in Minutes.
  */
  
    function getLocalTimeZoneShort(){
	  let timeZone = new Intl.DateTimeFormat('en-GB', { timeZoneName: 'short' })
		.formatToParts(new Date())
		.find(part => part.type == "timeZoneName")
		.value;
		//console.log("Short TZ: ",timeZone);
		//might return "GMT+8" so split at '+'
		tzname = timeZone.split("+");
		//console.log(tzname, tzname[0]);
		return tzname[0];
  }
  
      function getLocalTimeZoneLong(){
	  let timeZone = new Intl.DateTimeFormat('en-GB', { timeZoneName: 'long' })
		.formatToParts(new Date())
		.find(part => part.type == "timeZoneName")
		.value;
		//console.log("LongtimeZone: ",timeZone);
		const systemOptions = new Intl.DateTimeFormat().resolvedOptions();
		//console.log(systemOptions.timeZone); // e.g., "Europe/Brussels" or "Asia/Riyadh"
		return systemOptions.timeZone;
  }
  
    function doesdbexist(dbname){
		
		var dbexists=false;
		
		var request = window.indexedDB.open(dbname);
		request.onupgradeneeded = function(e) {
			var db = e.target.result;
			if (e.oldVersion == 0) {// if oldVersion is 0 then the access has created an empty database
		   db.close();//close it
		   //console.log("No old version of db deleting the dummy db" + e.oldVersion);
		   indexedDB.deleteDatabase(db);//delete it, create it later if required.
			} else {
			dbexists=true;
			}
		
			//console.log("Database "+dbname+" exists is: ",dbexists);
			
		}
		return dbexists;
	}
	
	
/*The JSON file is loaded as text and converted to an object with JSON.parse().
The function is declared async in order to be able to use await and thus wait until loading is finished to continue with the display of the contents.*/	
/* also need a version to load remote data.*/
async function loadJSON() {
  //var response = await fetch(fname)
  //var str = await response.text()
  //var data = JSON.parse(str)
  //use the JSON in local file: TZJSON
  var idb = await importIDB("TZDatabase", "TimeZones", TZJSON["timezones"]);
  //await displayIDB(idb, "TimeZones", "dump"); //display db data
}
/*
The parameters of the importIDB function are the name of the database, the name of the table (store), and an array containing the list of objects to be stored in the table.

The database is open with the open method of indexedDB. A table is created with the createObjectStore method in a function associated with the onupgradeneeded event.

The keyPath: "name" option is intended to set the "name" property, which is part of every object stored in the database, as the search key. This is equivalent to the primary column in SQL.

When the database is successfully created, the success event is triggered and the contents of the associated function executed. We then open a transaction to access the table and store the records.*/


function importIDB(dname, sname, arr) {
  return new Promise(function(resolve) {
    var r = window.indexedDB.open(dname)
    r.onupgradeneeded = function() {
      var idb = r.result
      var store = idb.createObjectStore(sname, {keyPath: "TZName"})
    }
    r.onsuccess = function() {
      var idb = r.result
        let tactn = idb.transaction(sname, "readwrite")
    	  var store = tactn.objectStore(sname)
        for(var obj of arr) {
          store.put(obj)
        }
        resolve(idb)
    }
    r.onerror = function (e) {
     alert("Enable to access IndexedDB, " + e.target.errorCode)
    }    
  })
}

/*Accessing a record*/
/*The database is opened. A new transaction is performed and the store.get() method finds the record whose key is given. This function returns a promise so that we can ensure that the data are available before displaying them. */

async function getIDB(dname, sname, key) {
  return new Promise(function(resolve) {
    var r = indexedDB.open(dname);
      r.onsuccess = function(e) {
        var idb = r.result;
        let tactn = idb.transaction(sname, "readonly");
        let store = tactn.objectStore(sname);
        let data = store.get(key);
        data.onsuccess = function() {
          resolve(data.result);
        };
        tactn.oncomplete = function() {
          idb.close();
        };
     };
  })
}

/*parsePosixTZ
parsePosixTZ accepts a POSIX time zone string and returns an object like this:

> parsePosixTZ('PST8PDT,M3.2.0,M11.1.0')
{ stdAbbr: 'PST',
  stdOffset: 480,
  dst: true,
  dstAbbr: 'PDT',
  dstOffset: 420,
  dstStart: { month: 3, week: 2, day: 0, hour: 2, minute: 0, second: 0 },
  dstEnd: { month: 11, week: 1, day: 0, hour: 2, minute: 0, second: 0 } }
  */
function POSIXtoKitST(strPOSIX){
	console.log("Convert POSIX", strPOSIX );
	//Arduino TimeChangeRule  {abbrev, week, dow, month, hour, offset};
	//var strKitst = "Z{DST, 5, 1, 03, 01, 060 }{STD, 5, 1, 10, 02, 000 }";
	//POSIX GMT0BST,M3.5.0/1,M10.5.0/2
	//mmm could build array then convert to string ??
	// or use a map
	//var arrDST = ['DST', 5, 1, 03, 01, 060];
	//var arrSTD = ['STD', 5, 1, 10, 02, 000];
	var arrPOSTZ = parsePosixTZ(strPOSIX);
	console.log(arrPOSTZ);
	//((x < 10) ? '000' : (x < 100) ? '00' : (x < 1000) ? '0' : '')+(+x)
	const dstOffset = ((arrPOSTZ.dstOffset<10)?'00':(arrPOSTZ.dstOffset<100)?'0':'')+(+arrPOSTZ.dstOffset);
	const stdOffset = ((arrPOSTZ.stdOffset<10)?'00':(arrPOSTZ.stdOffset<100)?'0':'')+(+arrPOSTZ.stdOffset);
	const dstMonth =  ((arrPOSTZ.dstStart.month<10)?'0':'')+(+arrPOSTZ.dstStart.month);
	const stdMonth =  ((arrPOSTZ.dstEnd.month<10)?'0':'')+(+arrPOSTZ.dstEnd.month);
	const dstHour =  ((arrPOSTZ.dstStart.hour<10)?'0':'')+(+arrPOSTZ.dstStart.hour);
	const stdHour =  ((arrPOSTZ.dstEnd.hour<10)?'0':'')+(+arrPOSTZ.dstEnd.hour);
	const dstDay = arrPOSTZ.dstStart.day+1;
	const stdDay = arrPOSTZ.dstEnd.day+1;
	var strKitST = 'Z{DST, '+arrPOSTZ.dstStart.week+', '+ dstDay +', '+dstMonth
	+', '+dstHour+', '+dstOffset+' }' //str.padStart(4, "0")
	+'{STD, '+arrPOSTZ.dstEnd.week+', '+ stdDay +', '+stdMonth
	+', '+stdHour+', '+stdOffset
	+' }';
	console.log(strKitST);
	return strKitST;
}
async function searchTZ() {
  var key = document.getElementById("TZName").value;
  var infos = await getIDB("TZDatabase", "TimeZones", key);
  var tzPosix = JSON.stringify(infos, null, ' ');
  document.getElementById("TZPosix").innerHTML = tzPosix;
  console.log("Convert: ",tzPosix);
  POSIXtoKitST(tzPosix);
}
/*The function below is for a demonstration purpose only, to verify that the filling is done properly. In an application, the database is supposed to contain thousands of records, so its contents are not displayed in bulk.
However, it is useful to see how cursor can be used to browse the contents of the database.*/
function displayIDB(idb, sname, id) {//database, store , id is element id
  let storage = document.getElementById(id);
  let tactn = idb.transaction(sname, "readonly");
  let osc = tactn.objectStore(sname).openCursor();
  osc.onsuccess = function(e) {
    let cursor = e.target.result
    if (cursor) {
      storage.innerHTML += "TZName " + cursor.value["TZName"] + " : " + "TZString "+cursor.value["ZString"] + "<br>";
      cursor.continue();
    }
  } 
  tactn.oncomplete = function() {
    idb.close();
  }
}	
